from typing import List

from viberbot.api.event_type import EventType as BaseEventType


class EventType(BaseEventType):
    @classmethod
    def all(cls) -> List[str]:
        return [
            cls.SEEN, cls.CONVERSATION_STARTED, cls.DELIVERED, cls.MESSAGE,
            cls.SUBSCRIBED, cls.UNSUBSCRIBED, cls.FAILED, cls.WEBHOOK
        ]

    @classmethod
    def not_message_events(cls) -> List[str]:
        return [
            cls.SEEN, cls.CONVERSATION_STARTED, cls.DELIVERED, cls.SUBSCRIBED,
            cls.UNSUBSCRIBED, cls.FAILED, cls.WEBHOOK
        ]

    @classmethod
    def message_status_update(cls) -> List[str]:
        return [cls.DELIVERED, cls.FAILED, cls.SEEN]

    @classmethod
    def subscriptions(cls) -> List[str]:
        return [cls.SUBSCRIBED, cls.UNSUBSCRIBED]

    @classmethod
    def chat(cls) -> List[str]:
        return [cls.CONVERSATION_STARTED, cls.MESSAGE, cls.SUBSCRIBED]



from typing import List


def snake_case_2_camel_case(name):
    words = name.split('_')
    return "".join(["{}{}".format(word[0].upper(), word[1:]) for word in words])


class DefaultEnum:
    @property
    @classmethod
    def default(cls):
        return cls._default

    @classmethod
    def is_default(cls, choice):
        return choice == cls._default or choice is None

    @classmethod
    def is_valid(cls, choice):
        return choice in cls.all


class BgMediaType(DefaultEnum):
    picture = 'picture'
    gif = 'gif'

    all = (picture, gif)
    _default = 'picture'


class TextVerticalAlign(DefaultEnum):
    top = 'top'
    middle = 'middle'
    bottom = 'bottom'

    all = (top, middle, bottom)
    _default = middle


class TextHorizontalAlign(DefaultEnum):
    left = 'left'
    center = 'center'
    right = 'right'

    all = (left, center, right)
    _default = center


class TextSize(DefaultEnum):
    small = 'small'
    regular = 'regular'
    large = 'large'

    all = (small, regular, large)
    _default = regular


class ActionType(DefaultEnum):
    reply = 'reply'
    open_url = 'open-url'
    none = 'none'

    all = (reply, open_url, none)
    _default = reply


class MediaType(DefaultEnum):
    picture = 'picture'
    gif = 'gif'

    all = (picture, gif)
    _default = 'picture'


class Button:
    def __init__(
            self,
            action_body: str,
            columns: int = 6,
            rows: int = 1,
            silent: bool = None,
            image: str = None,
            text: str = None,
            action_type: str = None,
            text_v_align: str = None,
            text_h_align: str = None,
            text_size: str = None,
            text_opacity: int = None,
            bg_media: str = None,
            bg_color: str = None,
            bg_media_type: str = None,
            bg_loop: bool = None,
            _utm: str = '?utm_campaign=viber&utm_medium=link&utm_source=social-networks'
    ):
        """
        :param action_body: ext for reply and none. ActionType or URL for open-url.
            See reply logic for more details. For ActionType reply - text For
            ActionType open-url - Valid URL. Max length: 250 characters
        :param action_type: Type of action pressing the button will perform.
            Reply - will send a reply to the PA. open-url - will open the
            specified URL and send the URL as reply to the PA. See reply
            logic for more details. https://viber.github.io/docs/tools/keyboards/#replyLogic
        :param columns: Button width in columns. Allowed 1-6 See keyboard
            design for more details https://viber.github.io/docs/tools/keyboards/#keyboardDesign
        :param rows: Button height in rows. Allowed 1-2. See keyboard design
            for more details https://viber.github.io/docs/tools/keyboards/#keyboardDesign
        :param silent: Determine whether the user action is presented in the
            conversation
        :param bg_color: Background color of button. Valid color HEX value
        :param bg_media:  URL for background media content (picture or gif). Will be
            placed with aspect to fill logic
        :param bg_media_type: picture, gif. For picture - JPEG and PNG files are
            supported. Max size: 500 kb
        :param bg_loop: When true - animated background media (gif) will loop
            continuously. When false - animated background media will play
            once and stop
        :param image: URL of image to place on top of background (if any). Can
            be a partially transparent image that will allow showing some of
            the background. Will be placed with aspect to fill logic
        :param text: Text to be displayed on the button. Can contain some HTML
            tags - see keyboard design for more details
        :param text_v_align: Vertical alignment of the text.
            Valid values: top, middle, bottom
        :param text_h_align: Horizontal align of the text.
        :param text_opacity: Text opacity 0-100
        """
        assert text or bg_media or image or bg_color, \
            "Button should have at least one of the following: text, bg_media, image, bg_color"

        self.action_body = action_body
        self.action_type = action_type

        assert columns is None or (0 < columns <= 6), 'columns possible values 1-6'
        self.columns = columns

        # assert columns is None or (0 < rows <= 2), 'rows possible values 1-2'
        self.rows = rows

        assert text_opacity is None or (0 < text_opacity <= 100), 'text opacity possible values 1-100'
        self.text_opacity = text_opacity

        self.silent = silent
        self.image = image
        self.text = text
        self.text_v_align = text_v_align
        self.text_h_align = text_h_align
        self.text_size = text_size

        # Background
        self.bg_media = bg_media
        self.bg_media_type = bg_media_type
        self.bg_loop = bg_loop
        self.bg_color = bg_color

        # Append utm
        self._utm = _utm
        if self.action_type == ActionType.open_url:
            self.action_body = "{}{}".format(self.action_body, self._utm)

    def to_dict(self) -> dict:
        d = {}

        mapping = {field: snake_case_2_camel_case(field) for field in Button.__init__.__annotations__.keys() if
                   not field.startswith('_')}
        for field_name, api_name in mapping.items():
            value = getattr(self, field_name)
            if value is not None:
                d[api_name] = value

        return d


class ExternalLinkButton(Button):
    def __init__(self, url: str, text: str, action_type=ActionType.open_url, **kwargs):
        super(ExternalLinkButton, self).__init__(
            action_body=url,
            text=text,
            action_type=action_type,
            **kwargs)


class Keyboard:
    def __init__(self, buttons: List[Button], default_height: bool = None, bg_color: str = None):

        """
        The public accounts API allows sending a custom keyboard with each
        messsage, to supply the user with a set of predefined replies or
        actions. The keyboard can be attached to any message type or sent on
        it’s on. Once received, the keyboard will appear to the user instead
        of the device’s native keyboard. The keyboards are fully customizable
        and can be created and designed specifically for the PA’s needs.

        See more https://viber.github.io/docs/tools/keyboards/#keyboards

        :param default_height: When true - the keyboard will always be
            displayed with the same height as the native keyboard.When false -
            short keyboards will be displayed with the minimal possible height.
            Maximal height will be native keyboard height.
        :param bg_color: Background color of the keyboard. Valid color HEX value
        :param buttons: Array containing all keyboard buttons by order.
        """
        self.default_height = default_height
        self._buttons = buttons.copy()
        self.bg_color = bg_color

    def add_button(self, button: Button) -> None:
        self._buttons.append(button)

    def to_dict(self) -> dict:
        d = {
            "Type": "keyboard",
            "Buttons": [button.to_dict() for button in self._buttons]
        }

        if self.bg_color:
            d['BgColor'] = self.bg_color
        if self.default_height:
            d["DefaultHeight"] = self.default_height

        return d


class Carousel:
    """
    The Carousel Content Message type allows a user to scroll through a list of
    items, each composed of an image, description and call to action button. 
    Use Carousel with Rich Media Messages.
    
    :param alt_text: Backward compatibility text, limited to 7,000 characters	 
    :param buttons_group_columns: Number of columns per carousel content block.
        Default 6 columns 1 - 6
    :param buttons_group_rows: Number of rows per carousel content block. 
        Default 7 rows 1 - 7
    :param buttons: Array of buttons. Max of 6 * ButtonsGroupColumns * ButtonsGroupRows
    """

    def __init__(
            self,
            buttons_group_columns: int = 6,  # ButtonsGroupColumns
            buttons_group_rows: int = 6,  # ButtonsGroupRows
            buttons: List[Button] = []
    ):
        assert 0 < buttons_group_rows <= 7, 'Possible values for buttons group rows 1 - 7'
        self.buttons_group_rows = buttons_group_rows

        assert 0 < buttons_group_columns <= 6, 'Possible values for buttons group columns 1 - 6'
        self.buttons_group_columns = buttons_group_columns

        self._buttons = buttons

    def to_dict(self) -> dict:
        return {
            'ButtonsGroupColumns': self.buttons_group_columns,
            'ButtonsGroupRows': self.buttons_group_rows,
            "Buttons": [button.to_dict() for button in self._buttons]
        }



from typing import List

from viberbot.api.messages import TextMessage, StickerMessage, ContactMessage, LocationMessage, VideoMessage, \
    URLMessage, RichMediaMessage
from viberbot.api.messages.data_types.contact import Contact
from viberbot.api.messages.data_types.location import Location
from viberbot.api.messages.message import Message
from viberbot.api.user_profile import UserProfile
from viberbot.api.viber_requests import ViberMessageRequest

from aioviber.api import Api
from aioviber.keyboard import Keyboard, Carousel


class Chat:
    def __init__(self, message: ViberMessageRequest = None, api: Api = None, sender: UserProfile = None):
        assert message or sender, 'Specify message or sender'

        self.message = message
        self.api = api
        self._sender = sender

    @property
    def sender(self):
        sender = self._sender
        try:
            sender = self.message.sender
        except AttributeError:
            pass
        return sender

    def send_messages(self, messages: List[Message]):
        return self.api.send_messages(
            to=self.sender.id,
            messages=messages
        )

    def send_message(self, message: Message):
        return self.api.send_message(
            to=self.sender.id,
            message=message
        )

    def send_text(self, text: str, keyboard: Keyboard = None, min_api_version=None, tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=TextMessage(
                text=text,
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            ))

    def send_sticker(self, sticker_id: int, keyboard: Keyboard = None, min_api_version=None, tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=StickerMessage(
                sticker_id=sticker_id,
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            )
        )

    def send_contact(self, name: str, phone_number: str, avatar: str = None, keyboard: Keyboard = None,
                     min_api_version=None, tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=ContactMessage(
                contact=Contact(name=name, phone_number=phone_number, avatar=avatar),
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            )
        )

    def send_location(
            self,
            lat: float,
            lon: float,
            keyboard: Keyboard = None,
            min_api_version=None,
            tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=LocationMessage(
                location=Location(lat, lon),
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            )
        )

    def send_video(
            self,
            media: str,
            size: int,
            thumbnail: str = None,
            duration: int = None,
            text: str = None,
            keyboard: Keyboard = None,
            min_api_version=None,
            tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=VideoMessage(
                media=media,
                size=size,
                thumbnail=thumbnail,
                duration=duration,
                text=text,
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            )
        )

    def send_url(
            self,
            url: str,
            keyboard: Keyboard = None,
            min_api_version=None,
            tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=URLMessage(
                media=url,
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            )
        )

    def send_rich_media(
            self,
            rich_media: Carousel,
            keyboard: Keyboard = None,
            min_api_version: int = 2,
            tracking_data: str = None):
        return self.api.send_message(
            to=self.sender.id,
            message=RichMediaMessage(
                rich_media=rich_media.to_dict(),
                keyboard=keyboard.to_dict() if keyboard else None,
                min_api_version=min_api_version,
                tracking_data=tracking_data
            )
        )



import logging
import os


def setup_logging():
    LOGGING_LEVEL = getattr(logging, os.environ.get('BOT_LOGGING_LEVEL', 'DEBUG'))
    logging.basicConfig(
        level=LOGGING_LEVEL,
        format='%(asctime)s | %(name)s | %(levelname)s - %(message)s'
    )



import asyncio as aio
from aioviber.middleware import signature_middleware

from aiohttp import web


async def ping(request):
    return web.Response(text='pong')


def get_app(bot, loop=None) -> web.Application:
    loop = aio.get_event_loop() if loop is None else loop
    app = web.Application(
        loop=loop,
        middlewares=[signature_middleware]
    )
    app.router.add_get('/ping', ping)
    app.router.add_static('/static', 'static')
    app.bot = bot
    return app



from aioviber.bot import Bot  # noqa
from aioviber.chat import Chat  # noqa
from aioviber.eventtype import EventType  # noqa
from aioviber.messagetype import MessageType  # noqa
from aioviber.api import Api, BotConfiguration  # noqa
from aioviber.keyboard import Keyboard, Button, ExternalLinkButton, Carousel  # noqa



import logging
import asyncio as aio
import re

import aiohttp
from aiohttp import web
from viberbot import BotConfiguration
from viberbot.api.viber_requests import ViberMessageRequest
from viberbot.api.viber_requests import ViberRequest, create_request
from urllib.parse import urlparse

from aioviber.app import get_app
from aioviber.chat import Chat
from aioviber.eventtype import EventType

from aioviber.api import Api
from aioviber.messagetype import MessageType

API_URL = "https://chatapi.viber.com/pa"
USER_AGENT = "WOWSViber/1.0"

__author__ = "Danil Ivanov"
__copyright__ = "Copyright 2017 Danil Ivanov"
__license__ = "MIT"

logger = logging.getLogger("aioviber")


class Bot:
    def __init__(self,
                 name: str,
                 avatar: str,
                 auth_token: str,
                 webhook: str,
                 webhook_events: list = None,
                 host: str = '0.0.0.0',
                 port: int = 8000,
                 loop: aio.AbstractEventLoop = None,
                 check_signature: bool = True):
        assert len(name) < 28, "Length of name should be shorty then 28 symbols"
        self.name = name
        self.avatar = avatar
        self.auth_token = auth_token

        # Server
        self.host = host
        self.port = port

        # Loop
        self.loop = aio.get_event_loop() if loop is None else loop

        # Viber API
        self._session = None
        self.api = Api(bot_configuration=BotConfiguration(
            name=self.name,
            avatar=self.avatar,
            auth_token=self.auth_token
        ), session=self.session, loop=self.loop)

        # Viber webhook
        self.webhook = webhook
        self.webhook_events = webhook_events

        def no_event_handle(event_type: str):
            return lambda msg: logger.debug("no event handle for %s", event_type)

        def no_message_handle(message_type: str):
            return lambda msg: logger.debug("no message handle for %s", message_type)

        # Callback — function for request processing messages excluded
        self._events_callbacks = {et: no_event_handle(et) for et in EventType.all()}
        # command — functions for processing text messages
        self._commands = []
        # handle — functions for processing messages exclude text
        self._handlers = {mt: no_message_handle(mt) for mt in MessageType.all()}

        self._default = lambda event: None

        # Application
        self.check_signature = check_signature
        self.app = self.get_app()

    @property
    def session(self):
        if not self._session:
            self._session = aiohttp.ClientSession()
        return self._session

    async def set_webhook_on_startup(self):
        await  aio.sleep(3)  # waiting while api will be available

        logger.info('set web hook on startup %s', self.webhook)

        self.loop.create_task(self.api.set_webhook(self.webhook, self.webhook_events))

    def get_app(self) -> web.Application:
        """
        Create aiohttp application for webhook handling
        """
        app = get_app(self, loop=self.loop)

        # webhook handler
        webhook_path = urlparse(self.webhook).path
        app.router.add_post(webhook_path, self.webhook_handle)

        # register webhook
        app.on_cleanup.append(lambda a: a.bot.api.unset_webhook())
        app.on_startup.append(lambda a: a.bot.set_webhook_on_startup())

        return app

    async def webhook_handle(self, request):
        data = await request.json()
        viber_request = create_request(data)  # type: ViberRequest

        t = self.loop.create_task(self._process_request(viber_request))
        # TODO add error processing

        return web.Response()

    async def _process_request(self, request: ViberRequest):
        logger.debug('request: %s', str(request))

        coro = None

        # Process request with function from _events_callbacks
        if request.event_type in EventType.all():
            coro = self._events_callbacks[request.event_type](request)

        # Process messages
        if request.event_type == EventType.MESSAGE:
            coro = self._process_message(request)

        if coro:
            # TODO: add error processing
            t = self.loop.create_task(coro)

    async def _process_message(self, request: ViberMessageRequest):
        logger.debug('_process_message %s', request)

        if request.message._message_type == MessageType.TEXT:  # isinstance
            # Process text messages by commands
            for pattern, handler in self._commands:
                matched = re.search(pattern, str(request.message.text), re.I)
                if matched:
                    return await handler(Chat(message=request, api=self.api), matched)
        else:
            # Process other messages types with _handlers
            return await self._handlers[request.message._message_type](Chat(request, self.api))

    def run(self):
        web.run_app(self.app, host=self.host, port=self.port, loop=self.loop)

    def __del__(self):

        try:
            self.session.close()
        except AttributeError:
            pass

    def add_command(self, regexp, fn):
        """ Register regexp based command for text messages processing """
        self._commands.append((regexp, fn))

    def command(self, regexp: str):
        """
        Register a new command.
        Command — coro for text message processing.
        :param: Regular expression matching the request with text message to register
        """

        def decorator(coro):
            assert aio.iscoroutinefunction(coro), 'Decorated function should be corountine'

            self.add_command(regexp, coro)
            return coro

        return decorator

    def add_handler(self, message_type, coro):
        self._handlers[message_type] = coro

    def message_handler(self, message_type):
        """
        Set handler for message type:
            - text
            - picture
            - contact
            - location
            - file
            - video
            - sticker
            - rich_media
            - url
        """

        def decorator(coro):
            assert aio.iscoroutinefunction(coro), 'Decorated function should be corountine'

            self.add_handler(message_type, coro)
            return coro

        return decorator

    def event_handler(self, event_type):
        """
        Set callback for specific event type:
            - delivered
            - seen
            - conversation_started
            - message
            - subscribed
            - unsubscribed
            - failed
            - webhook
        """
        assert event_type in EventType.not_message_events(), 'Wrong event type'

        def wrap(coro):
            assert aio.iscoroutinefunction(coro), 'Decorated function should be corountine'

            self._events_callbacks[event_type] = coro
            return coro

        return wrap

    def on_subscribed(self):
        def decorator(coro):
            assert aio.iscoroutine(coro), 'function should be corountine'

            self._events_callbacks[EventType.SUBSCRIBED] = coro
            return coro

        return decorator



from typing import List

from viberbot.api.messages import MessageType as BaseMessageType


class MessageType(BaseMessageType):
    @classmethod
    def all(cls) -> List[str]:
        return [
            cls.TEXT, cls.RICH_MEDIA, cls.STICKER, cls.URL, cls.LOCATION, cls.CONTACT, cls.FILE, cls.PICTURE, cls.VIDEO
        ]

    @classmethod
    def text(cls) -> List[str]:
        return [
            cls.TEXT, cls.URL
        ]



from setuptools import setup, find_packages
import os

from configparser import ConfigParser

PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))
PROJECT_INI = os.path.join(PROJECT_DIR, "project.ini")

config = ConfigParser()
config.read(PROJECT_INI)


def get_config(opt):
    return config.get("project", opt)


NAME = get_config('name')
DESCRIPTION = get_config('description')
URL = get_config('url')
AUTHOR = 'nonamenix'
AUTHOR_EMAIL = 'nonamenix@gmail.com'
README_TXT = 'README.txt'

REQUIREMENTS_FILE = 'requirements.txt'
REQUIREMENTS = open(os.path.join(PROJECT_DIR, REQUIREMENTS_FILE)).readlines()

VERSION = get_config('version')
DEV_VERSION = os.environ.get('DEV_VERSION')
if DEV_VERSION:
    VERSION = '{}.dev{}'.format(VERSION, DEV_VERSION)
    config.set("project", 'version', VERSION)
    with open(PROJECT_INI, 'w') as f:
        config.write(f)

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description='hello',
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    include_package_data=True,
    packages=find_packages(),
    install_requires=REQUIREMENTS,
)



import logging
from typing import List

import aiohttp
import asyncio as aio
from viberbot import BotConfiguration
from viberbot.api.consts import VIBER_BOT_API_URL, BOT_API_ENDPOINT
from viberbot.api.messages.message import Message

from aioviber.eventtype import EventType

default_headers = {
    'User-Agent': 'aioviber/1.0',
    'Accept-Encoding': 'gzip, deflate',
    'Accept': '*/*',
    'Connection': 'keep-alive',
}


class Api:
    endpoints = BOT_API_ENDPOINT

    def __init__(self,
                 bot_configuration: BotConfiguration,
                 session: aiohttp.ClientSession,
                 loop: aio.AbstractEventLoop):
        self._logger = logging.getLogger('aioviber.api')
        self._bot_configuration = bot_configuration
        self._viber_bot_api_url = VIBER_BOT_API_URL
        self.session = session
        self.loop = loop

    async def _make_request(self, endpoint: str, data: dict = {}):
        data['auth_token'] = self._bot_configuration.auth_token
        url = '{}/{}'.format(self._viber_bot_api_url, endpoint)
        async with self.session.post(url=url, json=data, headers=default_headers) as response:
            result = await response.json()

        if result['status'] != 0:
            error_text = "failed with result: {}".format(result)
            self._logger.error(error_text)
            raise Exception(error_text)

        return result

    async def send_message(self, to: str, message: Message):
        """ The send_message API allows PAs to send messages to Viber users
        who subscribe to the PA. Sending a message to a user will be possible
        only after the user has subscribed to the PA by pressing the subscribe
         button or by sending a message to the PA (see subscribed callback for
         additional information).

        The API supports a variety of message types: text, picture, video, file,
        location, contact and URL. Specific post data examples and required
        parameters for each message type are given below."""

        if not message.validate():
            error_text = "failed validating message: {0}".format(message)
            self._logger.error(error_text)
            raise Exception(error_text)

        data = message.to_dict()
        data.update({
            'receiver': to,
            'sender': {
                'name': self._bot_configuration.name,
                'avatar': self._bot_configuration.avatar
            }
        })
        result = await self._make_request(self.endpoints.SEND_MESSAGE, data)

        return result['message_token']

    async def set_webhook(self, url: object, webhook_events: object = None) -> object:
        """ For each set_webhook request Viber will send a callback to the
        webhook URL to confirm it is available. The expected HTTP response to
        the callback is 200 OK – any other response will mean the webhook is
         available. If the webhook is not available the set_webhook response
         sent to the user will be status 1: invalidUrl."""

        self._logger.debug(u"setting webhook to url: {0}".format(url))
        data = {
            'url': url
        }
        if webhook_events is not None:
            if isinstance(webhook_events, str):
                webhook_events = [webhook_events]

            for event in webhook_events:
                if event not in EventType.all():
                    webhook_events.pop(event)
                    self._logger.warning('Wrong event type {} in set_webhook'.format(event))

            data['event_types'] = webhook_events

        result = await self._make_request(self.endpoints.SET_WEBHOOK, data)
        return result['event_types']

    async def unset_webhook(self):
        """ Once you set a webhook to your PA your 1-on-1 conversation button
        will appear and users will be able to access it.
        At the moment there is no option to disable the 1-on-1 conversation
        from the PA settings, so to disable this option you’ll need to remove
        the webhook you set for the account. Removing the webhook is done by
        Posting a set_webhook request with an empty webhook string."""

        self._logger.debug("unsetting webhook")
        return await self.set_webhook('')

    async def send_messages(self, to: str, messages: List[Message]):
        self._logger.debug("going to send messages: {0}, to: {1}".format(messages, to))

        return await aio.gather(
            *[self.send_message(to, message) for message in messages], loop=self.loop
        )

    async def get_users_status(self, ids: List[str]):
        """ The get_online request will fetch the online status of a given
        subscribed PA members. The API supports up to 100 user id per request
        and those users must be subscribed to the PA."""
        assert isinstance(ids, list), 'ids is not instance of list'
        assert len(ids) != 0, 'ids shoudn\'t be empty list'

        result = await self._make_request(self.endpoints.GET_ONLINE, {'ids': ids})
        return result['users']

    async def get_user_details(self, user_id: str):
        """ The get_user_details request will fetch the details of a specific
        Viber user based on his unique user ID. The user ID can be obtained
        from the callbacks sent to the PA regrading user’s actions. This
        request can be sent twice during a 12 hours period for each user ID."""

        assert isinstance(user_id, str), 'wrong type of user_id'
        result = await self._make_request(self.endpoints.GET_USER_DETAILS, {'id': user_id})
        return result['user']

    async def get_account_info(self):
        return await self._make_request(self.endpoints.GET_ACCOUNT_INFO)



import hmac
import hashlib

import logging
from aiohttp import web
from aiohttp.web_request import Request

logger = logging.getLogger(__name__)


def calculate_message_signature(message, auth_token):
    return hmac.new(
        bytes(auth_token.encode('ascii')),
        msg=message,
        digestmod=hashlib.sha256).hexdigest()


def verify_signature(request_data, signature, auth_token):
    return signature == calculate_message_signature(request_data, auth_token)


async def signature_middleware(app, handler):
    """Check request signature"""

    async def middleware_handler(request: Request):
        if request.method == 'POST' and app.bot.check_signature:
            sig = request.query.get('sig')
            body = await request.read()
            if verify_signature(body, sig, app.bot.auth_token):
                response = await handler(request)
            else:
                logger.warning('Post requests with bad signature {sig} {body}'.format(
                    sig=sig, body=body
                ))
                response = web.HTTPForbidden()
        else:
            response = await handler(request)

        return response

    return middleware_handler
